/*
   Copyright (C) 2000  Ulric Eriksson <ulric@siag.nu>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the Licence, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "xfonts.h"	/* from common */
#include <X11/xpm.h>

#include "RulerP.h"

static float floatOne = 1.0;

#define offset(field) XtOffsetOf(RulerRec, ruler.field)
static XtResource resources[] = {
	{
		XtNleftMargin,
		XtCLeftMargin,
		XtRInt,
		sizeof(int),
		offset(left_margin),
		XtRImmediate,
		(XtPointer)72
	}, {
		XtNrightMargin,
		XtCRightMargin,
		XtRInt,
		sizeof(int),
		offset(right_margin),
		XtRImmediate,
		(XtPointer)72
	}, {
		XtNpaperWidth,
		XtCPaperWidth,
		XtRInt,
		sizeof(int),
		offset(paper_width),
		XtRImmediate,
		(XtPointer)595
	}, {
		XtNtabDistance,
		XtCTabDistance,
		XtRInt,
		sizeof(int),
		offset(tab_distance),
		XtRImmediate,
		(XtPointer)36
	}, {
		XtNrulerTopCol,
		XtCRulerTopCol,
		XtRInt,
		sizeof(int),
		offset(top_col),
		XtRImmediate,
		(XtPointer)0
	}, {
		XtNzoom,
		XtCZoom,
		XtRFloat,
		sizeof(float),
		offset(zoom),
		XtRFloat,
		(XtPointer)&floatOne
	}, {
		XtNfont,
		XtCFont,
		XtRFontStruct,
		sizeof(XFontStruct *),
		offset(font),
		XtRString,
		XtDefaultFont
	}
};
#undef offset

/* methods */
static void Realize(Widget w, XtValueMask *, XSetWindowAttributes *);
static void Redisplay(Widget, XEvent *, Region);
static Boolean SetValues(Widget, Widget, Widget, ArgList, Cardinal *);

/* actions */
static void RulerAction(Widget, XEvent *, String *, Cardinal *);

static XtActionsRec actions[] =
{
	{"ruler", RulerAction},
};

/* translations */
static char translations[] =
"<Key>:		ruler()\n";

RulerClassRec rulerClassRec = {
  { /* core fields */
    /* superclass		*/	(WidgetClass) &widgetClassRec,
    /* class_name		*/	"Ruler",
    /* widget_size		*/	sizeof(RulerRec),
    /* class_initialize		*/	NULL,
    /* class_part_initialize	*/	NULL,
    /* class_inited		*/	FALSE,
    /* initialize		*/	NULL,
    /* initialize_hook		*/	NULL,
    /* realize			*/	Realize,
    /* actions			*/	actions,
    /* num_actions		*/	XtNumber(actions),
    /* resources		*/	resources,
    /* num_resources		*/	XtNumber(resources),
    /* xrm_class		*/	NULLQUARK,
    /* compress_motion		*/	TRUE,
    /* compress_exposure	*/	TRUE,
    /* compress_enterleave	*/	TRUE,
    /* visible_interest		*/	FALSE,
    /* destroy			*/	NULL,
    /* resize			*/	NULL,
    /* expose			*/	Redisplay,
    /* set_values		*/	SetValues,
    /* set_values_hook		*/	NULL,
    /* set_values_almost	*/	XtInheritSetValuesAlmost,
    /* get_values_hook		*/	NULL,
    /* accept_focus		*/	NULL,
    /* version			*/	XtVersion,
    /* callback_private		*/	NULL,
    /* tm_table			*/	translations,
    /* query_geometry		*/	XtInheritQueryGeometry,
    /* display_accelerator	*/	XtInheritDisplayAccelerator,
    /* extension		*/	NULL
  },
  { /* ruler fields */
    /* empty			*/	0
  }
};

WidgetClass rulerWidgetClass = (WidgetClass)&rulerClassRec;


static void RulerAction(Widget w, XEvent *event, String *params, Cardinal *n)
{
	;
}

#define superclass (&coreClassRec)

static void Realize(Widget w, XtValueMask *v, XSetWindowAttributes *a)
{
	RulerWidget rw = (RulerWidget)w;
	unsigned long mask = 0;
	XGCValues values;
	(*superclass->core_class.realize)(w, v, a);
	values.font = rw->ruler.font->fid;
	mask = GCFont;
	rw->ruler.gc = XCreateGC(XtDisplay(w), XtWindow(w), mask, &values);
}

static void Redisplay(Widget w, XEvent *event, Region r)
{
	RulerWidget aw = (RulerWidget)w;
	int h = aw->core.height;
	int w1 = aw->core.width;
	Display *dpy = XtDisplay(w);
	Screen *s = XtScreen(w);
	Window wi = XtWindow(w);
	int i, o;
	GC gc = aw->ruler.gc;
	unsigned long black = BlackPixelOfScreen(s);
	unsigned long white = WhitePixelOfScreen(s);
	float zoom = aw->ruler.zoom;
	int pw, td, lm, rm, tc, ps, pe, in;

	XSetForeground(dpy, gc, white);

	pw = aw->ruler.paper_width;
	td = aw->ruler.tab_distance;
	lm = aw->ruler.left_margin;
	rm = aw->ruler.right_margin;
	tc = aw->ruler.top_col;
	ps = lm+tc+1;
	pe = pw-rm+tc;

	XFillRectangle(dpy, wi, gc, zoom*ps, 5, zoom*(pw-lm-rm), h-10);
	o = -1;

	XSetForeground(dpy, gc, black);

	/* draw the start for the first line in a paragraph */
	XDrawLine(dpy, wi, gc, zoom*ps-2+o, 0, zoom*ps+o, 7);
	XDrawLine(dpy, wi, gc, zoom*ps+o, 7, zoom*ps+2+o, 0);

	/* draw the start for subsequent lines */
	XDrawLine(dpy, wi, gc, zoom*ps-2+o, h, zoom*ps+o, h-7);
	XDrawLine(dpy, wi, gc, zoom*ps+o, h-7, zoom*ps+2+o, h);

	/* draw tab stops */
	for (i = ps+td; i < pe; i += td) {
		XDrawLine(dpy, wi, gc, zoom*i-2+o, h, zoom*i+o, h-7);
		XDrawLine(dpy, wi, gc, zoom*i+o, h-7, zoom*i+2+o, h);
	}

	/* draw half inch markers */
	for (i = ps+36; i < pe; i += 72) {
		XDrawLine(dpy, wi, gc, zoom*i+o, 9, zoom*i+o, h-10);
	}

	/* draw inch numbers */
	for (i = ps+72, in = 1; i < pe; i += 72, in++) {
		char num[10];
		int nw;
		sprintf(num, "%d", in);
		nw = XTextWidth(aw->ruler.font, num, strlen(num));
		XDrawString(dpy, wi, gc, zoom*i+o-nw/2, h-9, num, strlen(num));
	}
	XDrawLine(dpy, wi, gc, zoom*tc, 0, zoom*tc, h-1);
	XDrawLine(dpy, wi, gc, zoom*(pw+tc-1), 0, zoom*(pw+tc-1), h-1);
	XDrawLine(dpy, wi, gc, 0, 0, 0, h-1);
	XDrawLine(dpy, wi, gc, 0, 0, w1-1, 0);
	XDrawLine(dpy, wi, gc, 0, h-1, w1-1, h-1);
	XDrawLine(dpy, wi, gc, w1-1, 0, w1-1, h-1);
}

static Boolean SetValues(Widget current, Widget request, Widget new,
		ArgList args, Cardinal *nargs)
{
	RulerWidget rw1 = (RulerWidget)current;
	RulerWidget rw2 = (RulerWidget)new;
	Boolean do_redisplay =
			(rw1->ruler.tab_distance != rw2->ruler.tab_distance)
			|| (rw1->ruler.zoom != rw2->ruler.zoom)
			|| (rw1->ruler.top_col != rw2->ruler.top_col)
			|| (rw1->ruler.left_margin != rw2->ruler.left_margin)
			|| (rw1->ruler.right_margin != rw2->ruler.right_margin)
			|| (rw1->ruler.paper_width != rw2->ruler.paper_width);

	return do_redisplay;
}

void RulerSetZoom(Widget w, float zoom)
{
	RulerWidget rw = (RulerWidget)w;

	if (zoom < .1) zoom = .1;
	if (zoom > 10) zoom = 10;
	if (zoom != rw->ruler.zoom) {
		rw->ruler.zoom = zoom;
		XClearWindow(XtDisplay(w), XtWindow(w));
		Redisplay(w, NULL, None);
	}
}

